import unittest
from pyportify import views


class UserScopeTest(unittest.TestCase):

    def test_user_scope(self):
        scope = views.user_scope

        assert not scope.google_loggedin()
        assert not scope.spotify_loggedin()



#!/usr/bin/env python

from pyportify.wsgi import application
import cherrypy


def main():
    cherrypy.tree.graft(application, "/")
    cherrypy.server.unsubscribe()
    server = cherrypy._cpserver.Server()

    server.socket_host = "0.0.0.0"
    server.socket_port = 3132
    server.thread_pool = 10

    server.subscribe()

    cherrypy.engine.start()
    cherrypy.engine.block()

if __name__ == '__main__':
    main()



"""
Django settings for pyportify project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '3=95+i=l*s(ql1k8m+!s22e44w+yxbn^g2voq50rzxvltnr&ix'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.sessions',
    'django.contrib.staticfiles',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
)

ROOT_URLCONF = 'pyportify.urls'

WSGI_APPLICATION = 'pyportify.wsgi.application'


LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = False

USE_L10N = False

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(BASE_DIR, "pyportify", "static")

SPOTIFY_APPKEY = os.path.join(BASE_DIR, "pyportify", "spotify_appkey.key")


LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': ('%(asctime)s [%(process)d] [%(levelname)s] ' +
                       '%(filename)s:%(lineno)s:%(funcName)s %(message)s'),
            'datefmt': '%Y-%m-%d %H:%M:%S'
        },
    },
    'handlers': {
        'console': {
            'level': 'WARNING',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django.request': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': False,
        },
    }
}




from getpass import getpass
import json
import os
import sys

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "pyportify.settings")
import django
django.setup()

from pyportify import views as app

try:
    input = raw_input
except NameError:
    pass


def main():

    google_email = input("Enter Google email address: ")
    google_pass = getpass("Enter Google password: ")

    app.user_scope.google_login(google_email, google_pass)
    if not app.user_scope.google_loggedin():
        print "Invalid Google username/password"
        sys.exit(1)

    spotify_user = input("Enter Spotify username: ")
    spotify_pass = getpass("Enter Spotify password: ")

    app.user_scope.spotify_login(spotify_user, spotify_pass)
    if not app.user_scope.spotify_loggedin():
        print "Invalid Spotify username/password"
        sys.exit(1)

    playlists = app.spotify_playlists(None)
    playlists = json.loads(playlists.content)["data"]

    app.transfer_playlists(playlists)


if __name__ == '__main__':
    main()



from django.conf import settings
from django.conf.urls import patterns, url

print settings.STATIC_ROOT

urlpatterns = patterns('',
    url(r'^google/login$', 'pyportify.views.google_login'),
    url(r'^spotify/login$', 'pyportify.views.spotify_login'),
    url(r'^portify/transfer/start$', 'pyportify.views.transfer_start'),
    url(r'^spotify/playlists', 'pyportify.views.spotify_playlists'),

    url(r'^$', 'django.views.static.serve',
        {'document_root': settings.STATIC_ROOT, 'path': 'index.html'}),
    url(r'^(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': settings.STATIC_ROOT}),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages
import os

basepath = os.path.dirname(__file__)
readme_rst = os.path.join(basepath, "README.rst")
requirements_txt = os.path.join(basepath, "requirements.txt")

with open(readme_rst) as readme:
    long_description = readme.read()

with open(requirements_txt) as reqs:
    install_requires = [
        line for line in reqs.read().split('\n') if (line and not
                                                     line.startswith('--'))
    ]

setup(
    name='pyportify',
    version="0.1.10",
    author='Josh Braegger',
    author_email='rckclmbr@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/rckclmbr/pyportify',
    license='Apache 2.0',
    description='Django app to transfer your spotify playlists to Google Play '
                'Music',
    long_description=long_description,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
    entry_points={
        'console_scripts': [
            'pyportify = pyportify.server:main',
            'pyportify-copyall = pyportify.copy_all:main',
        ],
    },
    data_files=(
        ('', [
        "LICENSE.txt",
        ]),
    ),
    zip_safe=False,
    install_requires=install_requires,
)



import json
import threading
from django.conf import settings

from django.http import JsonResponse
from gmusicapi import Mobileclient
import spotify


class UserScope(object):

    def __init__(self):
        self.googleapi = Mobileclient()

        config = spotify.Config()
        config.load_application_key_file(settings.SPOTIFY_APPKEY)
        self.spotify_session = spotify.Session(config)
        self.loop = spotify.EventLoop(self.spotify_session)
        self.logged_in_event = threading.Event()
        self._google_loggedin = False

    def _logged_in_listener(self, session, error_type):
        if error_type is spotify.ErrorType.OK:
            # Now wait for the connection_state_listener to be state LOGGED_IN
            return
        # Otherwise, there's a login error and we can return immediately
        self.logged_in_event.set()

    def _connection_state_listener(self, session):
        if session.connection.state is spotify.ConnectionState.LOGGED_IN:
            self.logged_in_event.set()

    def start(self):
        self.loop.start()
        self.spotify_session.on(spotify.SessionEvent.LOGGED_IN,
                                self._logged_in_listener)
        self.spotify_session.on(spotify.SessionEvent.CONNECTION_STATE_UPDATED,
                                self._connection_state_listener)

    def google_login(self, username, password):
        self._google_loggedin = self.googleapi.login(username, password)
        return self._google_loggedin

    def spotify_login(self, username, password):
        self.logged_in_event.clear()
        user_scope.spotify_session.login(username, password)
        self.logged_in_event.wait()
        return self.spotify_session.user

    def spotify_loggedin(self):
        return self.spotify_session.user

    def google_loggedin(self):
        return self._google_loggedin


user_scope = UserScope()
user_scope.start()


def google_login(request):

    data = json.loads(request.body)

    email = data.get("email")
    password = data.get("password")

    user_scope.google_login(email, password)
    if not user_scope.google_loggedin():
        return JsonResponse(dict(
            status=400,
            message="login failed.",
        ))

    return JsonResponse(dict(
        status=200,
        message="login successful."
    ))


def spotify_login(request):

    data = json.loads(request.body)

    username = data.get("username")
    password = data.get("password")

    user_scope.spotify_login(username, password)
    if not user_scope.spotify_loggedin():
        return JsonResponse(dict(
            status=400,
            message="login failed.",
        ))
    return JsonResponse(dict(
        status=200,
        message="login successful."
    ))


def transfer_start(request):

    lists = json.loads(request.body)

    if not user_scope.google_loggedin():
        return JsonResponse({"status": 401, "message": "Google: not logged in."})

    if not user_scope.spotify_loggedin():
        return JsonResponse({"status": 402, "message": "Spotify: not logged in."})

    if not lists:
        return JsonResponse({"status": 403, "message": "Please select at least one playlist."})

    transfer_playlists(lists)
    return JsonResponse({"status": 200, "message": "transfer will start."})


def spotify_playlists(request):

    container = user_scope.spotify_session.playlist_container
    playlists = container.load()

    ret_playlists = [{
        "name": "Starred Tracks",
        "uri": "spotify:user:%s:playlist:starred"
               % user_scope.spotify_session.remembered_user_name,
    }]

    for playlist in playlists:
        if isinstance(playlist, spotify.PlaylistFolder):
            continue
        playlist.load()
        plist = {
            "name": playlist.name,
            "uri": playlist.link.uri,
        }
        ret_playlists.append(plist)

    return JsonResponse({"status": 200, "message": "ok", "data": ret_playlists})


def transfer_playlists(playlists):
    s = user_scope.spotify_session
    g = user_scope.googleapi
    for d_list in playlists:
        if ":starred" in d_list["uri"]:
            sp_playlist = s.get_starred()
            sp_playlist.name = 'Starred Tracks'
        else:
            sp_playlist = s.get_playlist(d_list["uri"])
        gm_track_ids = []

        print "Gathering tracks for playlist %s" % sp_playlist.name

        track_count = len(sp_playlist.tracks)
        for i, sp_track in enumerate(sp_playlist.tracks):
            sp_track.load()
            if sp_track.artists:
                sp_artist = sp_track.artists[0]
            else:
                sp_artist = None

            search_query = "%s - %s" % (sp_artist.name, sp_track.name)
            search_results = g.search_all_access(search_query, max_results=1)
            songs = search_results.get("song_hits")
            if songs:
                gm_track_id = songs[0]["track"]["nid"]
                gm_track_ids.append(gm_track_id)
                print "(%s/%s) Found '%s' in Google Music" % (i+1, track_count, search_query)
            else:
                print "(%s/%s) No match found for '%s'" % (i+1, track_count, search_query)

        # Once we have all the gm_trackids, add them
        if len(gm_track_ids) > 0:
            print "Creating in Google Music... ",
            playlist_id = g.create_playlist(sp_playlist.name)
            g.add_songs_to_playlist(playlist_id, gm_track_ids)
            print "Done"
